@@ -1,7 +1,7 @@
 package Simulation::Automate;
 
 use vars qw( $VERSION );
-$VERSION = "0.9.4";
+$VERSION = "0.9.6";
 
 #################################################################################
 #                                                                              	#
@@ -18,7 +18,7 @@ $VERSION = "0.9.4";
 #this module is then called via eval() and used by Simulation::Automate.pm 
 #Loops calls &Automate::main() at every pass through the loops.
 #
-#$Id: Automate.pm,v 1.3 2003/04/07 15:00:31 wim Exp $
+#$Id: Automate.pm,v 1.2 2003/09/04 09:53:25 wim Exp $
 #
 
 
@@ -28,6 +28,7 @@ use strict;
 use Cwd;
 use Exporter;
 use lib '.';
+use Simulation::Automate::Remote;
 
 @Simulation::Automate::ISA = qw(Exporter);
 @Simulation::Automate::EXPORT = qw(
@@ -36,7 +37,17 @@ use lib '.';
 		     &localinstall
                   );
 
+#===============================================================================
 sub synsim {
+  my $remotehost=&check_for_remote_host();
+  if($remotehost){
+    &run_on_remote_host($remotehost)
+  } else {
+    &run_local(); # new name for sub synsim
+  }
+}
+#===============================================================================
+sub run_local {
 my $datafile=shift||'synsim.data';
 
 ################################################################################
@@ -75,7 +86,7 @@ return 1;
 
 sub preprocess_commandline {
 my $datafile=$_[0];
-my ($batch,$interactive,$nosims,$plot,$verbose,$warn)=(0,0,0,0,0,0);
+my ($batch,$interactive,$nosims,$plot,$verbose,$warn,$justplot)=(0,0,0,0,0,0,0);
 my $default=1;
 if(@ARGV) {
 my $dtf=0;
@@ -85,11 +96,19 @@ my $dtf=0;
 	$_[0]=$_;$datafile=$_;$default=0;$dtf=0;
       }
       if(/-b/){$batch=1;next} 
-      if(/-i/){$interactive=1;next}
+      if(/-i/){$interactive=1;$plot=1;$verbose=1;next}
       if(/-N/){$nosims=1;next}
       if(/-p/){$plot=1;next}
       if(/-v/){$verbose=1;next}
       if(/-w/){$warn=1;next;}
+      if(/-P/){$justplot=1;next}
+      if(/-D/) {
+	(not -d 'TEMPLATES') && mkdir 'TEMPLATES';
+	(not -d 'TEMPLATES/SIMTYPES') && mkdir 'TEMPLATES/SIMTYPES';
+	(not -d 'TEMPLATES/DEVTYPES') && mkdir 'TEMPLATES/DEVTYPES';
+	(not -d 'SOURCES') && mkdir 'SOURCES';
+	die "An empty directory structure has been created\n";
+      }
       if(/-h|-\?/) { 
 my $script=$0;
 $script=~s/.*\///;
@@ -106,8 +125,8 @@ Possible switches:
 
 none: defaults to -f $datafile
  -f [filename]: 'file input'. Expects a file containing info about simulation and device type.
- -i : interactive. Creates a plot on the screen after every iteration.
- -p : plot.  Not fully implemented yet.
+ -i : interactive. Creates a plot on the screen after every iteration. Implies -p -v
+ -p : plot.  
  -v : 'verbose'. Sends simulator output to STDOUT, otherwise to simlog file
  -w : 'warnings'. Show warnings for undefined variables.
  -N : 'No simulations'. Does only postprocessing
@@ -122,8 +141,10 @@ my $test=$ARGV[@ARGV-1];
 if($test!~/^\-/) {
 $datafile=$test;
 $default=0;
+$_[0]=$datafile;
 }
 }
+
     if($default) {
 print STDERR "Assuming $datafile as input data filename\n" if $verbose;
 }
@@ -133,12 +154,26 @@ print STDERR "No command line arguments given. Assuming $datafile as input data
 
 if(not(-e "./TEMPLATES" && -d "./TEMPLATES" && -e "./$datafile")) {
 die  "
-The script must be executed in a device directory. 
-A device directory must contain a TEMPLATES subdir 
-with the device and analysis templates, and a data file
+The current directory must contain at least a TEMPLATES/SIMTYPE subdir with the simulation templates, and a data file. See documentation for more information.
+
+If Simulation::Automate is installed locally, the current directory must be in the same directoy as the Simulation directory.
+
 ";
 }
+if($justplot){
+#convenience function to plot results
+chomp(my $simtype=`egrep ^SIMTYPE $datafile`);
+$simtype=~s/^SIMTYPE\s*:\s*//;
+$simtype=~s/\s*$//;
+chomp(my $anatype=`egrep ^ANALYSIS_TEMPLATE $datafile`);
+$anatype=~s/^ANALYSIS_TEMPLATE\s*:\s*//;
+$anatype=~s/\s*$//;
+$datafile=~s/\.data//;
 
+chdir "${simtype}-$datafile";
+system("ggv ${simtype}-$anatype.ps");
+die "Done\n";
+}
 return [$batch,$interactive,$nosims,$plot,$verbose,$warn];
 } #END of preprocess_commandline
 
@@ -148,7 +183,6 @@ return [$batch,$interactive,$nosims,$plot,$verbose,$warn];
 # It gets back %data, which contains the list of parameters and their value-list for every simtype
 #
 
-
 sub generate_loop_module {
 my $specificref=shift; #this is the reference to %specific, the hash of arrays of data for each sim
 my $dataset=shift;
@@ -156,7 +190,6 @@ my $flagsref=shift;
 my ($batch,$interactive,$nosims,$plot,$verbose,$warn)=@{$flagsref};
 my $dataref=&fill_data_hash_multi($specificref);
 my %data=%{$dataref};
-
 open(MOD,">Loops_$dataset.pm");
 
 print MOD &strip(<<"ENDHEAD");
@@ -168,7 +201,7 @@ print MOD &strip(<<"ENDHEAD");
 *#                                                                              #
 *# Module to support script for SynSim simulations.                             #
 *# The subroutine in this module generates the loop to do multiple simulations. #
-*# This module is generated by Simulation::Automate.pm                                     #
+*# This module is generated by Simulation::Automate.pm                          #
 *#                                                                              #
 *################################################################################
 *
@@ -197,11 +230,19 @@ foreach my $sim (keys %data) {
 my $title=$data{$sim}{TITLE};
 delete $data{$sim}{TITLE};
 
+#experimental
+my $nruns=(exists $data{$sim}{NRUNS})?$data{$sim}{NRUNS}:1;
+if($nruns>1) {
+$data{$sim}{__NRUNS}=join(',',(1..$nruns));
+#also, make sure the SWEEPVAR list is comma-separated
+#$data{$sim}{$data{$sim}{SWEEPVAR}}=~s/;/,/g;
+}
 push @sims,$sim;
 print MOD &strip(<<"ENDSUBHEAD");
 
 *use lib '..','../..';
-*use Simulation::Automate::Main;
+*#use Simulation::Automate::Main;
+*use Simulation::Automate;
 *use Simulation::Automate::PostProcessors;
 *
 *sub execute_${sim}_loop {
@@ -217,6 +258,16 @@ print MOD &strip(<<"ENDSUBHEAD");
 
 ENDSUBHEAD
 
+if($data{$sim}{'PREPROCESSOR'}) {
+print MOD &strip('
+*my $preprocref=\&Simulation::Automate::PostProcessors::'.$data{$sim}{'PREPROCESSOR'}.';
+');
+} else {
+print MOD &strip('
+*my $preprocref;
+');
+}
+
 # TITLE is treated separately
 print MOD &strip(
 "*my \$TITLE = '$title';\n"
@@ -224,6 +275,9 @@ print MOD &strip(
 # define vars
  foreach my $par (sort keys %{$data{$sim}}) {
 if ($data{$sim}{$par}!~/,/) { # if just one item
+#WV21052003 support for "for..to..step.."-style lists
+$data{$sim}{$par}=&expand_list($data{$sim}{$par});
+
 $data{$sim}{$par}=~s/^\'//;
 $data{$sim}{$par}=~s/\'$//;
 print MOD &strip(
@@ -260,7 +314,7 @@ print MOD &strip('
 ');
   }
 
-  if ($data{$sim}{$par}!~/,/) { # if just one item
+  if ($data{$sim}{$par}!~/,/) { # if just one item, or it might be a sweep
     if($data{$sim}{$par}=~/(\d+)\s*\.\.\s*(\d+)/) {
       my $b=$1;
       my $e=$2;
@@ -274,7 +328,8 @@ print MOD &strip('
 ");
 
     } elsif($data{$sim}{$par}=~/;/) {
-      my $tmp=$data{$sim}{$par};
+     
+ my $tmp=$data{$sim}{$par};
       my $tmps=($tmp=~s/;/,/g);
       if($tmps>=$nsims){$nsims=$tmps+1}
       print MOD &strip(
@@ -311,6 +366,8 @@ print MOD &strip(
 foreach my $par (sort keys %{$data{$sim}}) {
 
   if ($data{$sim}{$par}=~/,/) { # if more than one item
+#WV21052003 support for "for to step"-style lists
+$data{$sim}{$par}=&expand_list($data{$sim}{$par});
 my $parlist=$data{$sim}{$par};
 $parlist=~s/,/ /g;
     print MOD &strip(
@@ -332,35 +389,55 @@ print MOD &strip('
 *	print RES "# Parameters for simulation run $i:\n";
 ');
 
+
+my $simtempl=$data{$sim}{SIMTYPE};
+my $anatempl=$data{$sim}{ANALYSIS_TEMPLATE}||'NoAnalysisDefined';
+my $subref=$anatempl;
+
+print MOD &strip('
+*my $resfilename="'.$sim.'-'.$anatempl.'";
+');
 foreach my $par (sort keys %{$data{$sim}}) {
 
   if ($data{$sim}{$par}=~/,/) { # if more than one item
-
     print MOD &strip(
-"*\$data{$par} = [\$$par];
+		     "*\$data{$par} = [\$$par];
 *\$sweepeddata{$par} = \$$par;
+*\$resfilename.=\"-${par}-\$$par\";
 *print STDERR \"# $par = \$$par\\n\" if \$v;
 *print RES \"# $par = \$$par\\n\";
 ");
-
   }
 }
 
+#WV21042003: old, sweep loops internal
 print MOD &strip(
 "* close RES;
+*\$resfilename.='.res';
+*#NEW01072003#rename \"\$dirname\/${sim}_C\$i.res\",\"\$dirname\/\$resfilename\";
 *my \$dataref = [\$nsims,\\\%data];
-*\$returnvalue=&main(\$dataset,\$i,\$dataref,\$flagsref);
+*\$returnvalue=&main(\$dataset,\$i,\$dataref,\$resfilename,\$flagsref);
 *
 ");
 
-my $simtempl=$data{$sim}{SIMTYPE};
-my $anatempl=$data{$sim}{ANALYSIS_TEMPLATE};
-my $subref=$anatempl;
+##WV21042003: new, sweep loops external
+#print MOD &strip(
+#"* close RES;
+#*my \$dataref = [\$nsims,\\\%data];
+#*my \$nsims=&pre_run(\$dataset,\$i,\$dataref,\$flagsref);
+#*foreach my \$simn (1..\$nsims) {
+#*\$returnvalue=&run(\$nsims,\$simn);
+#*}
+#*\$returnvalue=&post_run(\$dataset,\$i,\$dataref,\$flagsref);
+#*
+#*
+#");
+
 
 print MOD &strip(<<"ENDPP");
 *chdir \$dirname;
 *my \$dataref1 = [\$nsims,\\\%data,\\\%sweepeddata,\\\%last];
-*&Simulation::Automate::PostProcessors::$subref(\$dataset,\$i,\$dataref1,\$flagsref,\$returnvalue);
+*&Simulation::Automate::PostProcessors::$subref(\$dataset,\$i,\$dataref1,\$flagsref,\$returnvalue,\$preprocref);
 *chdir '..';
 ENDPP
 
@@ -473,18 +550,19 @@ $incl=~s/\s+$//;
 my @incl=($incl=~/,/)?split(/\s*,\s*/,$incl):($incl);
 foreach my $inclf (@incl) {
 open(INCL,"<$inclf")|| die $!;
-while(<INCL>){
-/^\s*\#/ && next;
-/^\s*$/ && next;
-chomp;
+while(my $incl=<INCL>) {
+$incl=~/^\s*\#/ && next;
+$incl=~/^\s*$/ && next;
+chomp $incl;
 # only configuration variables in include files!
-/:/ && do {push @common,$_};
+($incl=~/:/) && do {push @common,$incl};
 }
 close INCL;
 }
 }; # END of allow INCL files
-
- s/(\#.*)//;
+#print STDERR "$_\n";
+s/(\#.*)//;
+s/[;,]\s*$//; # be tolerant
 if(/SIMTYPE\s*:/) {
 my $sims=$_;
 (my $par,$simpatt)=split(/\s*:\s*/,$sims);
@@ -519,7 +597,33 @@ push @{$specific{$sim}},@common;
 
 return \%specific;
 } #END of allow_multiple_sims
-
+#-------------------------------------------------------------------------------
+#
+# this subroutine expands for-to-step lists in enumerated lists
+#
+sub expand_list {
+my $list=shift;
+my $sep=',';
+($list=~/;/)&&($sep=';');
+my @list=split(/\s*$sep\s*/,$list);
+if(@list==3 && $list!~/[a-zA-Z]/) { # 
+if(
+(abs($list[0])<abs($list[1]))&&(abs($list[2])<abs($list[1]-$list[0]))
+) { #it's a for-to-step list, expand it
+my $start=$list[0];
+my $stop=$list[1];
+my $step=$list[2];
+$list="$start";
+my $i=$start;
+#while($i*(abs($step)/$step)<$stop*(abs($step)/$step)){
+while(("$i" ne "$stop") && (abs($i)-abs($stop))<=0) { #yes, strange, but Perl says 0.9>0.9 is true!
+$i+=$step;
+$list.="$sep$i";
+}
+}
+}
+return $list;
+} # END of expand_list
 #===============================================================================
 
 sub execute_loop {
@@ -543,13 +647,23 @@ my $dev=${$simref}{$sim}->{DEVTYPE}||'';
 
 my $dirname= "${sim}-$dataset";
 
-  if(-e $dirname && -d $dirname && ($nosims==0)) {
+  if(-e $dirname && -d $dirname) {
+    if ($nosims==0) {
 print STDERR "\n# Removing old $dirname dir\n" if $verbose;
 if ($verbose) {
 print `rm -Rf $dirname`;
 } else {
 system("rm -Rf $dirname");
 }
+} else {
+print STDERR "\n# Cleaning up $dirname dir\n" if $verbose;
+if ($verbose) {
+print `rm -f $dirname/$sim-*`;
+print `rm -f $dirname/tmp*`;
+} else {
+system("rm -f $dirname/tmp*");
+}
+}
 }
 
   if (not -e "TEMPLATES/SIMTYPES/$sim$templ" ) {
@@ -577,7 +691,9 @@ print STDERR "No device template for $dev in TEMPLATES/DEVTYPES.\n" if $verbose;
 }
 # any file with this pattern is copied to the rundir.
 if (-d "SOURCES") {
+  if(<SOURCES/$sim*>){
 system("cp SOURCES/$sim* $dirname");
+}
 } 
 }
 print STDERR "#" x 80,"\n" if $verbose;
@@ -590,7 +706,439 @@ eval('&Loops_'.$dataset.'::execute_'.$sim.'_loop($dataset,\@flags);');
 } #sims
 return 1;
 } #END of &execute_loop
+#==============================================================================
+#Routines to support script for simulation automation.
+#The function &main() is called from the innermost loop of
+#Loops_*.pm
+push @Simulation::Automate::EXPORT, qw(
+		     main
+		     pre_run
+		     run
+		     post_run					
+);
 
+use  Simulation::Automate::Analysis;
+
+##################################################################################
+my $simpid=undef; 
+
+sub main {
+
+use Cwd;
+#&main($dataset,$i,$dataref,$resfilename,$flagsref);
+my $dataset=shift; 
+my $count=shift;
+my $dataref=shift;
+my $resfile=shift;
+my $flagsref=shift;
+
+my ($batch,$interactive,$nosims,$plot,$verbose,$warn)=@{$flagsref};
+
+(my $nsims, my $simdataref)=@{$dataref};
+
+print STDERR '#',"-" x 79, "\n" if $verbose;## Simulation run $count\n";
+
+my %simdata=%{$simdataref};
+
+my @results=();
+my $command=$simdata{COMMAND}||'perl inputfile outputfile'; 
+
+my $output_filter_pattern=$simdata{OUTPUT_FILTER_PATTERN}|| '.*';
+my $simtype=$simdata{SIMTYPE}||'';
+my $dirname= "${simtype}-$dataset";
+
+my $devtype=$simdata{DEVTYPE}||'';
+my $simtitle=$simdata{TITLE};
+my @sweepvarnames=();
+foreach my $key (keys %simdata) {
+($key!~/^_/) && next;
+($simtitle=~/$key/) && do {
+$simtitle=~s/$key/$key:$simdata{$key}->[0]/;
+};
+my $ndata=@{$simdata{$key}};
+if($ndata>1) {
+push @sweepvarnames,$key;
+  }
+}
+my $title="#$simtitle\n"||"#$devtype $simtype simulation\n";
+my $ext=$simdata{TEMPL}||'.templ';
+my $extin=$simdata{EXT}||'.pl';
+my $workingdir =cwd();
+chdir  "$workingdir/$dirname";
+
+## INPUT FILE CREATION
+
+foreach my $simn (1..$nsims) {
+  if($nsims==1){$simn=''} else {
+    print STDERR "# Subrun $simn of $nsims: " if $verbose;
+    foreach my $sweepvarname(@sweepvarnames){
+      print STDERR " $sweepvarname = ",$simdata{$sweepvarname}->[$simn-1] if $verbose;
+    }
+    print STDERR " \n" if $verbose;
+  }
+  my $inputfile= "${simtype}_${simn}$extin";
+  my $outputfile= "${simtype}_C${count}_${simn}.out";
+  my $commandline=$command;
+  $commandline=~s/inputfile/$inputfile/ig;
+  $commandline=~s/outputfile/$outputfile/ig;
+  
+  open (NEW, ">$inputfile");
+  print NEW ("$title");
+
+  foreach my $type ($devtype,$simtype) {
+    if($type) {
+      my $nsim=($simn eq '')?0:$simn;
+      &gen_sim_script ($nsim-1,"$simtype$ext",\%simdata,\*NEW,$dataset,$warn);
+      print NEW ("\n");
+    }
+  } # device and simulation templates
+  close (NEW);
+  
+  if($nosims==0) {
+    if($verbose) {
+      if (!defined($simpid = fork())) {
+	# fork returned undef, so failed
+	die "cannot fork: $!";
+      } elsif ($simpid == 0) {
+	# fork returned 0, so this branch is the child
+	exec("$commandline");
+	# if the exec fails, fall through to the next statement
+	die "can't exec $commandline : $!";
+      } else { 
+	# fork returned neither 0 nor undef, 
+	# so this branch is the parent
+	waitpid($simpid, 0);
+      } 
+      #system("$commandline");
+    } else { # not verbose
+      print STDERR "\n" if $verbose;
+      #      print STDERR grep /$output_filter_pattern/,`$commandline > simlog 2>&1`;
+      #or, with a pipe:
+      $simpid = open(SIM, "$commandline 2>&1 |") || die "can't fork: $!"; 
+      open(LOG,">simlog");
+      while (<SIM>) {
+	print LOG;
+	/$output_filter_pattern/ && do {
+	  print STDERR;# if $verbose;
+	};
+      } # while simulation is running
+      close LOG;
+      my $ppid=getpgrp($simpid);
+      if(not $ppid) {
+	close SIM || die "Trouble with $commandline: $! $?";
+      }
+      print STDERR "\n" if $verbose;
+    } #verbose or not
+# } # if simulations not disabled
+
+  if($nsims>1) {
+    #Postprocessing
+    &egrep($output_filter_pattern,"${simtype}_C${count}_${simn}.out",'>>',"${simtype}_C${count}_.out");
+  }
+  } # if simulations not disabled
+  my $i=($nsims>1)?$simn-1:0;
+  open(RES,"<${simtype}_C${count}_${simn}.out");
+  $results[$i]=<RES>;
+  my $another=<RES>; # This takes the next line, if any,
+  if($another) { # and if there is one, it assigns the filename to $results[$i]
+    $results[$i]="${simtype}_C${count}_${simn}.out";
+  }
+  close RES;
+} # nsims 
+
+#Postprocessing after sweep
+#if($nosims==0) {
+&egrep($output_filter_pattern, "${simtype}_C${count}_.out", '>>', "${simtype}_C$count.res");
+#NEW01072003#&egrep($output_filter_pattern, "${simtype}_C${count}_.out", '>>', $resfile);
+#}
+chdir "$workingdir";
+
+return \@results; # PostProcessors are only called after &main() exits.
+} #END of main()
+
+#========================================================================================
+#  NEW IMPLEMENTATION TO ALLOW POSTPROCESSING AFTER EVERY ELEMENT IN SWEEP
+#========================================================================================
+my @results=();
+my %simdata=();
+my $simtype='NO_SIMTYPE';
+my $dataset='NO_DATASET';
+my $count=0;
+my ($batch,$interactive,$nosims,$plot,$verbose,$warn);
+my $output_filter_pattern= '.*';
+my $command='perl inputfile outputfile'; 
+my $dirname= 'NO_DIRNAME';
+my $devtype='NO_DEVTYPE';
+my $simtitle='NO_TITLE';
+my $title="#$devtype $simtype simulation\n";
+my $ext='.templ';
+my $extin='.pl';
+my $workingdir = 'NO_WORKINGDIR';
+#------------------------------------------------------------------------------
+#&main(\$dataset,\$i,\$dataref,\$flagsref);
+#&pre_run(\$dataset,\$i,\$dataref,\$flagsref);
+#&run(\$dataset,\$i,\$dataref,\$flagsref);
+#&post_run(\$dataset,\$i,\$dataref,\$flagsref);
+
+sub pre_run {
+
+use Cwd;
+
+$dataset=shift; 
+$count=shift;
+my $dataref=shift;
+my $flagsref=shift;
+($batch,$interactive,$nosims,$plot,$verbose,$warn)=@{$flagsref};
+
+(my $nsims, my $simdataref)=@{$dataref};
+
+print STDERR '#',"-" x 79, "\n" if $verbose;## Simulation run $count\n";
+
+%simdata=%{$simdataref};
+#my @results=();
+$command=$simdata{COMMAND}||'perl inputfile outputfile'; 
+
+$output_filter_pattern=$simdata{OUTPUT_FILTER_PATTERN}|| '.*';
+$simtype=$simdata{SIMTYPE}||'';
+ $dirname= "${simtype}-$dataset";
+ $devtype=$simdata{DEVTYPE}||'';
+ $simtitle=$simdata{TITLE};
+foreach my $key (keys %simdata) {
+($key!~/^_/) && next;
+($simtitle=~/$key/) && do {
+$simtitle=~s/$key/$key:$simdata{$key}/;
+};
+}
+ $title="#$simtitle\n"||"#$devtype $simtype simulation\n";
+ $ext=$simdata{TEMPL}||'.templ';
+ $extin=$simdata{EXT}||'.pl';
+ $workingdir =cwd();
+chdir  "$workingdir/$dirname";
+return $nsims;
+} #END of pre_run()
+#------------------------------------------------------------------------------
+sub run {
+
+my $nsims=shift;
+my $simn=shift;
+
+#use Cwd;
+#my ($nsims, my $simdataref)=@{$dataref};
+
+## INPUT FILE CREATION
+
+#foreach my $simn (1..$nsims) {
+  if($nsims==1){$simn=''} else {
+    print STDERR "# Subrun $simn of $nsims \n" if $verbose;
+  }
+  my $inputfile= "${simtype}_${simn}$extin";
+  my $outputfile= "${simtype}_C${count}_${simn}.out";
+  my $commandline=$command;
+  $commandline=~s/inputfile/$inputfile/ig;
+  $commandline=~s/outputfile/$outputfile/ig;
+  
+  open (NEW, ">$inputfile");
+  print NEW ("$title");
+
+  foreach my $type ($devtype,$simtype) {
+    if($type) {
+      my $nsim=($simn eq '')?0:$simn;
+      &gen_sim_script ($nsim-1,"$simtype$ext",\%simdata,\*NEW,$dataset,$warn);
+      print NEW ("\n");
+    }
+  } # device and simulation templates
+  close (NEW);
+  
+  if($nosims==0) {
+    if($verbose) {
+      if (!defined($simpid = fork())) {
+	# fork returned undef, so failed
+	die "cannot fork: $!";
+      } elsif ($simpid == 0) {
+	# fork returned 0, so this branch is the child
+	exec("$commandline");
+	# if the exec fails, fall through to the next statement
+	die "can't exec $commandline : $!";
+      } else { 
+	# fork returned neither 0 nor undef, 
+	# so this branch is the parent
+	waitpid($simpid, 0);
+      } 
+      # system("$commandline");
+    } else { # not verbose
+      print STDERR "\n" if $verbose;
+      #      print STDERR grep /$output_filter_pattern/,`$commandline > simlog 2>&1`;
+      #or, with a pipe:
+      $simpid = open(SIM, "$commandline 2>&1 |") || die "can't fork: $!"; 
+      open(LOG,">simlog");
+      while (<SIM>) {
+	print LOG;
+	/$output_filter_pattern/ && do {
+	  print STDERR;# if $verbose;
+	};
+      } # while sinulation is running
+      close LOG;
+      my $ppid=getpgrp($simpid);
+      if(not $ppid) {
+	close SIM || die "Trouble with $commandline: $! $?";
+      }
+      print STDERR "\n" if $verbose;
+    } #verbose or not
+#  } # if simulations not disabled
+  if($nsims>1) {
+    #Postprocessing
+    &egrep($output_filter_pattern,"${simtype}_C${count}_${simn}.out",'>>',"${simtype}_C${count}_.out");
+  }
+  } # if simulations not disabled
+  my $i=($nsims>1)?$simn-1:0;
+  open(RES,"<${simtype}_C${count}_${simn}.out");
+  $results[$i]=<RES>;
+  my $another=<RES>; # This takes the next line, if any,
+  if($another) { # and if there is one, it assigns the filename to $results[$i]
+    $results[$i]="${simtype}_C${count}_${simn}.out";
+  }
+  close RES;
+#} # nsims 
+#no need to return @results, it's a package global now. Maybe return $results[$i], makes more sense.
+#return \@results; # PostProcessors are only called after &main() exits.
+return $results[$i]; # PostProcessors are only called after &main() exits.
+} # END of run()
+#------------------------------------------------------------------------------
+sub post_run {
+  if($nosims==0){
+#Postprocessing after sweep
+&egrep($output_filter_pattern, "${simtype}_C${count}_.out", '>>', "${simtype}_C$count.res");
+}
+chdir "$workingdir";
+
+return \@results; # PostProcessors are only called after &main() exits.
+
+} # END of post_run()
+#==============================================================================
+
+#print STDERR "\n","#" x 80,"\n#\t\t\tSynSim simulation automation tool\n#\n#  (c) Wim Vanderbauwhede 2000,2002-2003. All rights reserved.\n#  This program is free software; you can redistribute it and/or modify it\n#  under the same terms as Perl itself.\n#\n","#" x 80,"\n";
+
+#-------------------------------------------
+# SUBROUTINES used by main, pre_run, run, post_run
+#-------------------------------------------
+
+#--------------------------------------
+# GENERATION OF THE SIMULATION SCRIPT
+#--------------------------------------
+
+#WV What happens: the templates for _SIMTYPE  are read in
+#WV and the variables are substituted with the values from the .data file
+
+sub gen_sim_script {
+
+my $nsim=shift;
+my $templfilename=shift;
+my $simdataref=shift;
+my %simdata=%{$simdataref};
+my $fh=shift; 
+my $dataset=shift;
+my $warn=shift;
+my %exprdata=();
+my %keywords=();
+foreach my $key ( sort keys %simdata) {
+#  ($key!~/^_/) && next;
+#make sure substitutions happen in keyword values too
+  if ($key!~/^_/ ) {
+    if( $simdata{$key}=~/^_/) {
+my $parameter=$simdata{$key};
+${$keywords{$parameter}}{$key}=1;
+}
+next;
+}
+
+  if(@{$simdata{$key}}==1) {
+    $exprdata{$key}=&check_for_expressions(\%simdata,$key,$nsim);
+    foreach my $keyword (keys %{$keywords{$key}}) {
+
+$simdata{$keyword}=$exprdata{$key};
+
+}
+  } # if..else
+} # foreach 
+
+	# OPEN TEMPLATE
+	open (TEMPL, "<$templfilename")||die "Can't open $templfilename\n";
+
+	while (my $line = <TEMPL>) {
+
+		  foreach my $key (keys %simdata) {
+		    ($key!~/^_/) && next;
+			my $ndata=@{$simdata{$key}};
+			if($ndata>1) {
+			  if($line =~ s/$key(?!\w)/$simdata{$key}->[$nsim]/g){
+#			  print STDERR "# $key = ",$simdata{$key}->[$nsim],"\n" if $warn;
+			}
+			} else {
+#			  my $simdata=&check_for_expressions(\%simdata,$key,$nsim);
+			  #A dangerous addidtion to make SynSim handle words
+			  $exprdata{$key}||=$simdata{$key}->[0];
+			  $line =~ s/$key(?!\w)/$exprdata{$key}/g;
+			  #print STDERR "# $key = ",$simdata{$key}->[0],"\nLINE:$line\n" if $warn;
+			} # if..else
+
+		      } # foreach 
+
+		  # check for undefined variables
+		  while($line=~/\b(_\w+?)\b/&&$line!~/$1\$/) {
+		    my $nondefvar=$1;
+		    $line=~s/$nondefvar/0/g; # All undefined vars substituted by 0
+		    print STDERR "\nWarning: $nondefvar ($templfilename) not defined in $dataset.\n" if $warn; 
+		  } # if some parameter is still there
+		  print $fh $line;
+		} # while
+close TEMPL;
+
+} # END OF gen_sim_script 
+#------------------------------------------------------------------------------
+sub egrep {
+my $pattern=shift;
+my $infile=shift;
+my $mode=shift;
+my $outfile=shift;
+open(IN,"<$infile");
+open(OUT,"$mode$outfile");
+print OUT grep /$pattern/,<IN>;
+
+close IN;
+close OUT;
+}
+#------------------------------------------------------------------------------
+sub check_for_expressions {
+my $dataref=shift;
+my $key=shift;
+my $nsim=shift;
+my %simdata=%{$dataref};	
+my $expr=$simdata{$key}->[0];
+if($expr=~/(_[A-Z_]+)/) { # was "if"
+while($expr=~/(_[A-Z_]+)/) { # was "if"
+#variable contains other variables
+#_A =3*log(_B)+_C*10-_D**2
+#_A =3 ;log;_B;;_C;10;_D;;2
+my @maybevars=split(/[\ \*\+\-\/\^\(\)\[\]\{\}\?\:\=\>\<]+/,$expr);
+my @vars=();
+foreach my $maybevar ( @maybevars){
+($maybevar=~/_[A-Z]+/)&& push @vars,$maybevar;
+}
+foreach my $var (@vars) {
+my $simn=(@{$simdata{$var}}==1)?0:$nsim;
+$expr=~s/$var/$simdata{$var}->[$simn]/g;
+}
+}
+#print STDERR "$key=$expr=>",eval($expr),"\n";
+}
+return eval($expr);
+}
+
+################################################################################
+#
+# These routines are not used by synsim
+# They are used by make install
+#
 ################################################################################
 
 # Create simulation directory etc.
@@ -657,13 +1205,13 @@ mkdir "$synsimroot/Simulation", 0755;
 mkdir "$synsimroot/Simulation/Automate", 0755;
 }
   if(-d "Automate") {  
-foreach my $module (qw(Main PostProcessors Dictionary)){
+foreach my $module (qw(PostProcessors Dictionary)) {
 if( -e "Automate/$module.pm"){
 copy("Automate/$module.pm", "$synsimroot/Simulation/Automate/$module.pm");
 }
 }
-if($full){
-  foreach my $module (qw(Main PostProcLib Analysis)){
+if($full) {
+  foreach my $module (qw(Remote PostProcLib Analysis)){
 if( -e "Automate/$module.pm"){
 copy("Automate/$module.pm", "$synsimroot/Simulation/Automate/$module.pm");
 }
@@ -727,17 +1275,17 @@ SynSim is a generic template-driven simulation automation tool. It works with an
 =over
 
 =item 1.
-Download the gzipped tar file F<Simulation-Automate-0.9.4.tar.gz>
+Download the gzipped tar file F<Simulation-Automate-0.9.5.tar.gz>
 
 =item 2.
 Extract the archive:
 
-	tar -xvf Simulation-Automate-0.9.4.tar.gz
+	tar -xvf Simulation-Automate-0.9.5.tar.gz
 
 =item 3.
 Create the Makefile:
 
-	cd Simulation-Automate-0.9.4
+	cd Simulation-Automate-0.9.5
 	perl Makefile.PL
 
 =item 4.
@@ -781,7 +1329,7 @@ The archive structure is as follows:
 	Makefile.PL	  
         Automate.pm
 	Automate/
-                Main.pm
+                Remote.pm
         	PostProcLib.pm
                 Analysis.pm
 		Dictionary.pm
@@ -858,7 +1406,7 @@ B<NOTE:>
 SynSim creates a run directory ath the same level as the SOURCES and TEMPLATES directories. All commands (compilations etc.) are executed in that directory. As a consequence, paths to source files (e.g. header files) should be "C<../SOURCES/>I<sourcefilename>".
 
 
-=head2 datafile
+=head2 Datafile
 
 The datafile is the input file for synsim. It contains the list of simulation variables and their values to be substituted in the template files, as well as a number of configuration variables (See L<DATAFILE DESCRIPTION> for more information).
 
@@ -897,6 +1445,8 @@ Configuration variables ("keywords") are in UPPERCASE, and must be separated fro
 
 Lists of values have one or more items. Valid list separators are ',',';' and, for a 2-element list, '..'.
 
+If a (','- or ';'-separated) list has 3 elements START,STOP,STEP |START|<|STOP| and |STEP|<|STOP-START|, then this list will be expanded as a for-loop from START to STOP with step STEP.
+
 =item Section headers for multiple simulation types
 
 These must be lines containing only the simulation type 
@@ -930,10 +1480,10 @@ If more than one ';'-separated list exists, they must have the same number of it
 
 Example:
 
-	_PAR1 = 0;1;2
-	_PAR2 = 3;4;5
+	_PAR1 = 0;1;2;4
+	_PAR2 = 3;4;5;6
 
-defines 3 simulations: (_PAR1,_PAR2)=(0,3);(1,4);(2,5)
+defines 3 simulations: (_PAR1,_PAR2)=(0,3);(1,4);(2,5);(4,6)
 
 Values from ';'-separated lists are processed one after another while are values for all others parameters are kept constant. In other words, the ';'-separated list is the innermost of all nested loops.
 
@@ -947,6 +1497,16 @@ Example: consider simulation of packet loss vs number of buffers with 3 types of
 
 This will produces 6 files, each file containing the simulation results for all values of _NBUFS. A plot of this simulation would show a set of 6 curves, with _NBUFS as X-axis variable.
 
+=item Semicolon-separated from;to;step list
+
+This is a special case where the list has exactly three elements From;To;Step and following condition holds:
+
+  (|From|<|To|) AND (|Step|<|To-From|)
+
+Example:
+
+       _NBUFS = 16;64;8 #  from 16 to 64 in steps if 8: 16;24;32;40;48;56;64
+
 =item Double dotted list
 
 This is a shorthand for a ';'-separated list if the value increase in steps of 1. E.g. 0..7 is equivalent to 0;1;2;3;4;5;6;7.
@@ -999,6 +1559,10 @@ The name of the variable to be sweeped. Mandatory if the postprocessing routine
 
 The name of the variable to normalise the results with. The results will be divided by the corresponding value of the variable.
 
+=item NRUNS (optional)
+
+The number of times the simulation has to be performed. For statistical work.
+
 =item DATACOL (optional)
 
 The column in the output file which contains simulation results (default: 2). Mandatory if using any of the generic postprocessing routines. 
@@ -1072,32 +1636,55 @@ Postprocessing of the simulation results is handled by routines in the C<PostPro
 
 =head2 PostProcessors
 
-Routines to perform analysis on the simulation results in the PostProcessors module. In general you will have to create your own routines, but the version of C<PostProcessors.pm> in the distribution contains a number of more or less generic postprocessing routines:
+Routines to perform analysis on the simulation results in the PostProcessors module. In general you will have to create your own routines, but the version of C<PostProcessors.pm> in the distribution contains a number of more or less generic postprocessing routines. All of these have hooks for simple functions that modify a file in-place. To call these functions, include them in the datafile with the C<PREPROCESSOR> variable. e.g:
+
+  PREPROCESSOR : modify_results_file
+
+All functions must be put in PostProcessors.pm, and the template could be like this:
+
+  sub modify_results_file {
+  my $resultsfile=shift;
+  
+  open(IN,"<$resultsfile");
+  open(TMP,">$resultsfile.tmp");
+  while(<IN>) {
+  #Do whatever is required
+  }
+  close IN;
+  close TMP;
+  rename "$resultsfile.tmp","$resultsfile" or die $!;
+  }
 
 =over
 
 =item SweepVar
 
-Creates a plot using C<SWEEPVAR> as X-axis and all other variables as paramters. This routine is completely generic. The C<SWEEPVAR> value list must be semicolon-separated.
+Required configuration variables: C<SWEEPVAR>
 
-=item ErrorFlags 
+Creates a plot using C<SWEEPVAR> as X-axis and all other variables as parameters. This routine is completely generic. The C<SWEEPVAR> value list must be semicolon-separated.
 
-Creates a plot using C<SWEEPVAR> as X-axis and all other variables as paramters. Calculates average and 95% confidence intervals for C<_POP_SAMPLE> samples and plots error flags. This routine is almost fully generic, apart from the 95% interval which is fixed for now. See eg/ErrorFlags.data for an example datafile. The C<SWEEPVAR> value list must be comma-separated (!). The number of samples of the population C<_POP_SAMPLE> is a simulation variable rather than a configuration variable. As a consequence, it must be specified as a list, e.g. 
+=item SweepVarCond
 
- _POP_SAMPLE = 1..20
+Required configuration variables: C<SETVAR>,C<SWEEPVAR> and C<CONDITION>. 
 
-and not
+Creates a plot using C<SETVAR> as X-axis; C<SWEEPVAR> is checked against the condition C<COND> (or C<CONDITION>). The first value of C<SWEEPVAR> that meets the condition is plotted. All other variables are parameters. This routine is completely generic. The C<SWEEPVAR> value list must be semicolon-separated.
 
- _POP_SAMPLE = 20
+=item ErrorFlags
+
+Required configuration variables: C<SWEEPVAR>, C<NRUNS>
+
+Optional configuration variables: C<NSIGMAS>
+
+Creates a plot using C<SWEEPVAR> as X-axis and all other variables as paramters. Calculates average and 95% confidence intervals for C<NRUNS> simulation runs and plots error flags. This routine is fully generic, the confidence interval (95% by default) can be set with NSIGMAS. See eg/ErrorFlags.data for an example datafile. The C<SWEEPVAR> value list must be semicolon-separated.
 
 =item Histogram
 
+Required configuration variables: C<NBINS>
+
+Optional configuration variables: C<BINWIDTH>, C<OUTPUT_FILTER_PATTERN>, C<NSIGMAS>
+
 Creates a histogram of the simulation results. This requires the simulator to produce raw data for the histograms in a tabular format. The configuration variable C<OUTPUT_FILTER_PATTERN> can be used to 'grep' the simulator output. When specifying logscale X or XY for the plot, the histogram bins will be logarithmic. See eg/Histogram.data for an example. 
-The number of bins in the histogram must be specified via C<SWEEPVAR>, e.g.
-
- SWEEPVAR : _NBINS
-
- _NBINS = 20
+The number of bins in the histogram must be specified via C<NBINS>. The width of the bins can be set with C<BINWIDTH>.
 
 =back
 
@@ -1111,7 +1698,7 @@ Following functions are exported:
 			   &gnuplot		# idem. Just pipes the first argument string to gnuplot. The option -persist can be added to keep the plot window after gnuplot exits.
 			   &gnuplot_combined	# See example, most useful to create nice plots. Looks for all files matching ${simtempl}-${anatempl}-*.res, and creates a line in the gnuplot script based on a template you provide.	
 
-Following variables are exported:
+Following variables are exported (see PostProcLib.pm for a full list):
 
 			   %simdata		# contains all simulation variables and their value lists
 			   @results		# an array of all results for a sweep (i.e. a var with a ';'-sep. value list
@@ -1266,23 +1853,38 @@ NOTE: Because the extra arguments are last, the $title and $log arguments can no
 =head1 RUNNING SYNSIM
 
 The SynSim script must be executed in a subdirectory of the SynSim
-directory which contains the TEMPLATES subdir and the datafile (like the Example directory in the distribution).
+directory which contains the TEMPLATES subdir and the datafile (like the Example directory in the distribution). 
 
 The command line is as follows:
 
-	./synsim [-h -i -p -w -v -N -f datafile]
+	./synsim [-h -i -p -w -v -N -P -f] [datafile] [remote hostname]
 
 The C<synsim> script supports following command line options:
 
 	none: defaults to -f synsim.data
 	 -f [filename]: 'file input'. Expects a file containing info about simulation and device type. 
 	 -p : plot. This enables generation of postscript plots via gnuplot. A postprocessing routine is required to generate the plots.
-	 -i : interactive. Enables generation of a plot on the screen after every iteration. Assumes -p.  A postprocessing routine is required to generate the plots.
+	 -i : interactive. Enables generation of a plot on the screen after every iteration. Assumes -p -v.  A postprocessing routine is required to generate the plots.
 	 -v : 'verbose'. Sends simulator output to STDOUT, otherwise to the [rundir]/simlog file
 	 -w : 'warn'. Show warnings about undefined variables
 	 -N : 'No simulations'. Perform only postprocessing.
+	 -P : 'Plot only'. Only plots the generated PostScript file. 
 	 -h, -? : short help message
 
+If [remote hostname] is provided, SynSim will try to run the simulation on the remote host.
+
+The current implementation requires:
+
+-ssh access to remote host
+
+-scp access to remote host
+
+-rsync server on the local host
+
+-or,alternatively, an NFS mounted home directory
+
+-as such, it will (probably) only work on Linux and similar systems
+
 =head1 OUTPUT FILES
 
 SynSim creates a run directory C<{SIMTYPE}->I<[datafile without .data]>. It copies all necessary template files and source files to this directory; all output files are generated in this directory.
@@ -1521,19 +2123,7 @@ To access the binary files, you'll have to write your own postprocessing routine
 
 =head1 TO DO
 
-This module is still Alpha, a lot of work remains to be done to make it more user-friendly. The main tasks:
-
-=over
-
-=item *
-
-Add a GUI. A prototype can be found on my web site, it is already useful bu too early to include here.
-
-=item *
-
-The syntax for variables for use in the postprocessing modules should be redefined. At the moment, some of them are treated as configuration variables, some as simulation variables. I think a separate class of user-defined variables would be better.
-
-=back
+This module is still Alpha, a lot of work remains to be done to make it more user-friendly. The main tasks is to add a GUI. A prototype can be found on my web site, it is already useful but too early to include here. The next version will also make it easier to create your own postprocessing routines.
 
 =head1 AUTHOR
 
